//===-- Time.h --------------------------------------------------*- C++ -*-===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

// Include system time headers, adding missing functions as necessary

#ifndef LLDB_HOST_TIME_H
#define LLDB_HOST_TIME_H

#include <ctime>

#endif // LLDB_HOST_TIME_H
